/*
 * Copyright (c) 2007 Takuya SHIOZAKI <tshiozak@bsdclub.org>,
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <Windows.h>
#include <string.h>
#include "copyleft-vst.h"

AudioEffect *
createEffectInstance(audioMasterCallback audioMaster)
{
	return new CopyLeftVST(audioMaster);
}

// ----------------------------------------------------------------------

CopyLeftVST::CopyLeftVST(audioMasterCallback amc)
	: AudioEffectX(amc, 1, 1)
{
	this->setNumInputs(2);
	this->setNumOutputs(2);
	this->setUniqueID(0x7F7F7F00); //XXX
	this->canProcessReplacing();
	this->canDoubleReplacing();
}

CopyLeftVST::~CopyLeftVST()
{
}

void CopyLeftVST::setProgram(VstInt32)
{
}

void CopyLeftVST::setProgramName(char *name)
{
}

void CopyLeftVST::getProgramName(char *name)
{
	vst_strncpy(name, "dummy", kVstMaxProgNameLen);
}

void CopyLeftVST::getParameterLabel(VstInt32 index, char *label)
{
	vst_strncpy(label, "dummy", kVstMaxParamStrLen);
}

void CopyLeftVST::getParameterDisplay(VstInt32 index, char *text)
{
	vst_strncpy(text, "0", kVstMaxParamStrLen);
}

void CopyLeftVST::getParameterName(VstInt32 index, char *label)
{
	vst_strncpy(label, "default", kVstMaxParamStrLen);
}

void CopyLeftVST::setParameter(VstInt32 index, float value)
{
}

float CopyLeftVST::getParameter(VstInt32 index)
{
	return 0.0;
}

bool CopyLeftVST::getOutputProperties(VstInt32 index, VstPinProperties *props)
{
	switch (index) {
	case 0:
	case 1:
		vst_strncpy(props->label, "Left", 63);
		vst_strncpy(props->shortLabel, "Left", 63);
		props->flags = kVstPinIsActive | kVstPinIsStereo;
		props->arrangementType = kSpeakerArrStereo;
		return true;
	}
	return false;
}

bool CopyLeftVST::getInputProperties(VstInt32 index, VstPinProperties *props)
{
	switch (index) {
	case 0:
		vst_strncpy(props->label, "Left", 63);
		vst_strncpy(props->shortLabel, "Left", 63);
		props->flags = kVstPinIsActive | kVstPinIsStereo;
		return true;
	case 1:
		vst_strncpy(props->label, "Right", 63);
		vst_strncpy(props->shortLabel, "Right", 63);
		props->flags = kVstPinIsActive | kVstPinIsStereo;
		return true;
	}
	return false;
}

bool CopyLeftVST::getProgramNameIndexed(VstInt32 cat, VstInt32 index, char *text)
{
	if (index == 0) {
		vst_strncpy(text, "default", kVstMaxProgNameLen);
		return true;
	}
	return false;
}

bool CopyLeftVST::getEffectName(char *name)
{
	vst_strncpy(name, "CopyLeftVST", kVstMaxEffectNameLen);
	return true;
}

bool CopyLeftVST::getVendorString(char *text)
{
	vst_strncpy(text, "AoiMoe", kVstMaxVendorStrLen);
	return true;
}

bool CopyLeftVST::getProductString(char *text)
{
	vst_strncpy(text, "CopyLeft VST", kVstMaxProductStrLen);
	return true;
}

VstInt32 CopyLeftVST::getVendorVersion()
{
	return 1000;
}

void CopyLeftVST::processReplacing(float **inputs, float **outputs,
				   VstInt32 num_samples)
{
	memcpy(outputs[0], inputs[0], num_samples*sizeof (**inputs));
	memcpy(outputs[1], inputs[0], num_samples*sizeof (**inputs));
}

void CopyLeftVST::processDoubleReplacing(double **inputs, double **outputs,
					 VstInt32 num_samples)
{
	memcpy(outputs[0], inputs[0], num_samples*sizeof (**inputs));
	memcpy(outputs[1], inputs[0], num_samples*sizeof (**inputs));
}
